/*

highlight v4m1

Highlights arbitrary terms.

<http://johannburkard.de/blog/programming/javascript/highlight-javascript-text-higlighting-jquery-plugin.html>

MIT license.

Modified by Penagwin to allow custom css classes.
<dev.penagw.in>

Johann Burkard
<http://johannburkard.de>
<mailto:jb@eaio.com>

MODIFIED BY ESKO:
- change functionname
- convert to ES module
- fix removeHightlight
- fix removeHighlightText and highlightText to allow for nested highlights
- allow highlighting across newlines

*/

import jQuery from "jquery";
import escapeStringRegexp from "escape-string-regexp";

jQuery.fn.highlightText = function(pat, classn) {
	if (!classn) {
		classn = "highlightText";
	}

	function innerHighlight(node, pat) {
		var skip = 0;
		if (node.nodeType == 3) {
			var pos = node.data.toUpperCase().search(escapeStringRegexp(pat).replace(/\s/g, "\\s"));
			if (pos >= 0) {
				var spannode = document.createElement("span");
				spannode.className = String(classn);
				var middlebit = node.splitText(pos);
				var endbit = middlebit.splitText(pat.length);
				var middleclone = middlebit.cloneNode(true);
				spannode.appendChild(middleclone);
				middlebit.parentNode.replaceChild(spannode, middlebit);
				skip = 1;
			}
		} else if (node.nodeType == 1 && node.childNodes && !/(script|style)/i.test(node.tagName)) {
			for (var i = 0; i < node.childNodes.length; ++i) {
				i += innerHighlight(node.childNodes[i], pat);
			}
		}
		return skip;
	}
	return this.length && pat && pat.length
		? this.each(function() {
				this.normalize();
				innerHighlight(this, pat.toUpperCase());
		  })
		: this;
};

jQuery.fn.removeHighlightText = function(classn) {
	if (!classn) {
		classn = "highlightText";
	}
	return this.find("span." + classn)
		.contents()
		.unwrap();
};

export default jQuery;
