/**
 * @author WIPE
 * @module WebViewer3D
 */

define([
    "backbone"
], function (
    BackBone
)
{
    /**
     * @class URLUtils
     * This class is a singleton containing helper functions for URL handling.
     */
    const URLUtils = BackBone.Model.extend({
        /**
         * @returns true if the given string is a URL, else false
         *
         * @param {String} __string URL to the scene data
         */
        IsURL: function (__string)
        {
            if (!typeof __string === "string")
                LogMessage.Warning("IsURL expects a String parameter.");

            const regexprelative = /.\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;
            return (regexprelative.test(__string) || this.IsAbsoluteURL(__string));
        },

        /**
         * @returns true if the given string is a URL, else false
         *
         * @param {String} __string URL to the scene data
         */
        IsRelativeURL: function (__string)
        {
            return this.IsURL(__string) && !this.IsAbsoluteURL(__string);
        },

        /**
         * @returns true if the given string is a URL, else false
         *
         * @param {String} __string URL to the scene data
         */
        IsAbsoluteURL: function (__string)
        {
            const regexpabsolute = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;
            return regexpabsolute.test(__string);
        },

        /**
         * @returns the parent URL (i.e. the URL before the last forward slash) of __url.
         *          When there is no slash in __url then this function returns an empty string.
         * @param {String} __url Should be a URL string.
         */
        GetParentURL: function (__url)
        {
            if (!this.IsURL(__url))
                LogMessage.Warning("GetParentURL expects that its argument is a URL.");

            return __url.substring(0, __url.lastIndexOf("/"));
        },

        /**
         * Function to convert an URL to a string that can be used as a key in a map
         * Replaces # with / and chops off http, https and ftp
         */
        URLToKeyString: function(__url)
        {
            let keyStr = __url.replace("#", "/");
            keyStr = keyStr.replace(/(ftp|http|https):\/\/?/, "");
            return keyStr;
        }
    });

    return new URLUtils;

});
