import $ from "jquery";
import _ from "underscore";
import "./finishingprofileselector.css";

export default class FinishingProfileSelector {
	constructor(f, options) {
		this.f = f;
		this.options = options;
		this.select = $("<select>", {
			class: "finishingprofileselector",
			on: {
				change: e => {
					if (options && options.onChange) {
						options.onChange(e.target.value);
					}
				}
			}
		});
		this.element = $("<div>", {
			class: "hiddenElement"
		})
			.append(
				$("<span>", {
					class: "alignLabel",
					text: f.t("FinishingProfile")
				})
			)
			.append(
				$("<div>", {
					class: "offsetFields"
				}).append(this.select)
			);

		this._loadOperations = {};
	}

	hide() {
		this.element.addClass("hiddenElement");
	}

	show() {
		let identifier = (this.options && this.options.getIdentifier && this.options.getIdentifier()) || 0;
		let loadParameters = (this.options && this.options.getLoadParameters && this.options.getLoadParameters()) || null;
		if (!this._loadOperations[identifier]) {
			this._loadOperations[identifier] = this.f.getDataAdapter().loadFinishingProfiles(loadParameters);
		}
		return new Promise(resolve => {
			this._loadOperations[identifier].then(finishingProfiles => {
				this.select.empty();
				if (finishingProfiles.length > 0) {
					this.select.append(
						$("<option/>", {
							value: "",
							text: this.f.t("FinishingProfileChoose")
						})
					);
					_.each(finishingProfiles, finishingProfile => {
						this.select.append(
							$("<option/>", {
								value: finishingProfile.id,
								text: finishingProfile.name
							})
						);
					});
					resolve(true);
				} else {
					resolve(false);
				}
			});
		}).then(shouldShow => {
			if (shouldShow) {
				this.element.removeClass("hiddenElement");
			} else {
				this.hide();
			}
		});
	}

	canBeShown(activePage, overlayPage) {
		if (this.options.canBeShown && !this.options.canBeShown(activePage)) {
			return false;
		}

		if (!overlayPage) {
			return true;
		} else {
			return !!this.f.getDataAdapter().loadFinishingProfiles && overlayPage.is3D();
		}
	}

	setValue(finishingProfile) {
		this.select.val(finishingProfile);
	}
}
