import $ from "jquery";
import _ from "underscore";
import "./printablepartselector.css";
import View3DConstants from "../../view3D/View3DConstants";

export default class PrintablePartSelector {
	constructor(f, options) {
		this.f = f;
		this.options = options;
		this.select = $("<select>", {
			class: "printablepartselector",
			on: {
				change: e => {
					if (options && options.onChange) {
						if (e.target.value) {
							options.onChange(JSON.parse(e.target.value));
						} else {
							options.onChange();
						}
					}
				}
			}
		});
		this.element = $("<div>")
			.append(
				$("<span>", {
					class: "alignLabel",
					text: f.t("PrintablePart")
				})
			)
			.append(
				$("<div>", {
					class: "offsetFields"
				}).append(this.select)
			);
	}

	hide() {
		this.element.addClass("hiddenElement");
	}

	show(page) {
		this.element.removeClass("hiddenElement");

		// Prevent select being populated multiple times
		if (this._lastPage === page) {
			return this._previousLoadOperation;
		}
		this._lastPage = page;

		// Show loading...
		this.select.empty();
		this.select.append(
			$("<option/>", {
				text: this.f.t("PrintablePartLoading")
			})
		);
		this.select.prop("disabled", true);

		// Populate select with printable parts
		this._previousLoadOperation = page.getPrintableParts().then(result => {
			this.select.empty();
			this.select.prop("disabled", false);
			_.each(result, (printablePart, key) => {
				if (printablePart.FrontMaterial !== printablePart.BackMaterial) {
					this.select.append(
						$("<option/>", {
							value: JSON.stringify([key, View3DConstants.frontMaterialSelector]),
							text: `${printablePart.UserVisibleName}`
						})
					);

					this.select.append(
						$("<option/>", {
							value: JSON.stringify([key, View3DConstants.backMaterialSelector]),
							text: `${printablePart.UserVisibleName} ${this.f.t("PrintablePartBack")}`
						})
					);
				} else {
					this.select.append(
						$("<option/>", {
							value: JSON.stringify([key, View3DConstants.frontMaterialSelector]),
							text: `${printablePart.UserVisibleName}`
						})
					);
				}
			});

			if (this.select.children().length === 0) {
				this.select.prop("disabled", true);
				this.select.append(
					$("<option/>", {
						value: "",
						text: this.f.t("PrintablePartEmpty")
					})
				);
			} else {
				this.select.prepend(
					$("<option/>", {
						value: "",
						text: this.f.t("PrintablePartChoose"),
						selected: true
					})
				);
			}
		});

		return this._previousLoadOperation;
	}

	canBeShown(page) {
		return !!page.getPrintableParts;
	}

	setValue(requestedSelectedValue) {
		this.select.val(JSON.stringify(requestedSelectedValue));

		// Check if requested value actually exists, if not fire change event
		if (
			(this.select.val() && !requestedSelectedValue) ||
			(!this.select.val() && requestedSelectedValue) ||
			(this.select.val() && requestedSelectedValue && this.select.val() !== JSON.stringify(requestedSelectedValue))
		) {
			this.select.val("").trigger("change");
		}
	}

	getValue() {
		return this.select.val() ? JSON.parse(this.select.val()) : undefined;
	}
}
