import LayoutConstants from "@esko/web-viewer/layout/LayoutConstants";
import LayoutMode from "@esko/web-viewer/layout/LayoutMode";
import CoreConstants from "@esko/web-viewer/core/CoreConstants";
/**
 * @exports AnnotationViewMode
 * @memberof Layout
 * @class
 * Mode for mobile layout, shows the annotation widget
 * @constructor
 * @augments LayoutMode
 * @param {Core.ViewerFacade} f
 * @param {Layout} layout
 */
var AnnotationViewMode = function(f, layout) {
	LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.MOBILE_ANNOTATION);

	this._updateModeAvailability();
	this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(this._updateModeAvailability, this);
};

AnnotationViewMode.prototype = new LayoutMode();
AnnotationViewMode.prototype.constructor = AnnotationViewMode;

/**
 * Turns On/Off the layoutMode
 * @param {Boolean} toggle
 */
AnnotationViewMode.prototype.toggle = function(toggle) {
	if (toggle === undefined) {
		toggle = !this._isActive;
	}
	this._isActive = toggle;
	if(this._isActive) {
		this.f.emit("toolManager.activateTool", CoreConstants.TOOLS.ANNOTATION_TOUCH_DRAW);		
	} else {
		this.f.emit("toolManager.activeTool", (tool) => {
			if(tool.getIID() === CoreConstants.TOOLS.ANNOTATION_TOUCH_DRAW) {
				this.f.emit("toolManager.deactivateTool", CoreConstants.TOOLS.ANNOTATION_TOUCH_DRAW);		
			}
		})
	}
};
AnnotationViewMode.prototype.isModePanelExpanded = function() {
	if(this._layout.getMainContentElement().width() < 768) {
		// on small screens we do not want the mode panel to showup directly as this hides most of the view
		return false; 
	} else {
		return LayoutMode.prototype.isModePanelExpanded.call(this);
	}
}
AnnotationViewMode.prototype._updateModeAvailability = function() {
	var page = this.f.getDocumentController().getActivePage();
	this.f.emit("layout.modeAvailabilityChanged", this, page && !page.is3D(), {
		icon: "icon-WCR_Annotate",
		title: "AnnToolsTitle"
	});
};

/**
 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
 *
 * @param {String} itemname
 * @returns {Boolean}
 */
AnnotationViewMode.prototype.isItemAllowed = function(itemname) {
	// only allow viewport widget
	return (
		itemname === LayoutConstants.Widget.VIEWPORT ||
		itemname === LayoutConstants.Widget.ANNOTATION ||
		itemname === LayoutConstants.ToolbarComponent.DOCUMENTSELECTOR ||
		itemname === LayoutConstants.ToolbarComponent.APPROVAL ||
		itemname === LayoutConstants.ToolbarComponent.ANNOTATION ||
		itemname === LayoutConstants.Widget.MESSAGE
	);
};

/**
 * Checks if the state of the layout should be persisted.
 * @return {Boolean}
 */
AnnotationViewMode.prototype.isPersisted = function() {
	return false;
};

/**
 * Decide whether or not to show the mode panel
 * @override
 */
AnnotationViewMode.prototype.isModePanelEnabled = function() {
	return true;
};

/**
 * @override
 */
AnnotationViewMode.prototype.showModePanelWidgetTitles = function() {
	return false;
};

/**
 * list of needed widgets in modepanel for annotation view mode
 * @returns {Array}
 * @override
 */
AnnotationViewMode.prototype.getModePanelContent = function() {
	return [LayoutConstants.Widget.ANNOTATION];
};

export default AnnotationViewMode;
