import AbstractTool from "@esko/web-viewer/viewport/tools/AbstractTool";
import ShapeUtils from "@esko/shapeutils";
import ConversionConstants from "../utils/ConversionConstants";
import { inverse } from "./TextTransformation";
import CompareConstants from "@esko/web-viewer/compare/CompareConstants";

/**
 * Abstract class for tools that handle text using the TextSelectionModel.
 * TextSelectionModel uses document coordinates and not world/scene coordinates.
 * This class adds some methods to easily switch between scene and text coordinates
 *
 */
export default class extends AbstractTool {
	constructor(f, IID, textSelectionModel) {
		super(f, IID);
		this._textSelectionModel = textSelectionModel;
	}

	/**
	 * Gets a point in text coordinates for a mouse event
	 */
	getTextPoint(e) {
		var point = this.getWorldPoint(e);
		return inverse(this._textSelectionModel.getPage()).transformedPoint(point);
	}

	/**
	 * Gets the point inside the world in text coordinates for a mouse event
	 */
	getTextPointKeepInWorld(e) {
		var point = this.keepPointWithinWorldBounds(this.getWorldPoint(e));
		return inverse(this._textSelectionModel.getPage()).transformedPoint(point);
	}

	/**
	 * returns a charachter which is identified as 'close'
	 * @param {ShapeUtils.Point} point in text coordinates
	 */
	getCloseCharacter(loc) {
		var px = 5 * ConversionConstants.MM_TO_PX_RATIO;
		var d = this.sceneToWorldSize(new ShapeUtils.Size(px, px)).width;
		return this._textSelectionModel.getCharacter(loc, d);
	}

	/**
	 * By default text tools cannot be interrupted as the selection from the selection layer will be hidden if deactived.
	 */
	canBeInterrupted() {
		return false;
	}

	/**
	 * Text Tools only work in current and difference modes
	 */
	canBeActivated() {
		var compareModule = this.f.getModule("compare");
		if (compareModule) {
			var mode = compareModule.getActiveMode();
			var allowedModes = [CompareConstants.mode.SHOW_CURRENT, CompareConstants.mode.SHOW_DIFFERENCE];
			return !mode || allowedModes.indexOf(mode.getName()) !== -1;
		}
		return true;
	}
}
