import TextSearchModel from "@esko/web-viewer/svg/TextSearchModel";
import _ from "underscore";

class MultiPageTextSearchModel extends TextSearchModel {
	constructor(f, channelName, textModel, matchingPages) {
		super(f, channelName, textModel);

		this._matchingPages = matchingPages;
	}

	_getCurrenPageIndex() {
		const page = this.f
			.getDocumentController()
			.getActivePage()
			.getPagenum();

		return this._matchingPages.indexOf("" + page) !== -1 ? this._matchingPages.indexOf("" + page) : -1;
	}

	_getPreviousMatchingPage() {
		if (this._getCurrenPageIndex() === -1) {
			const currentPage = this.f
				.getDocumentController()
				.getActivePage()
				.getPagenum();
			const matchingPagesCopy = [...this._matchingPages];
			const previousPage = _.find(matchingPagesCopy.reverse(), pageStr => {
				return parseInt(pageStr, 10) < currentPage;
			});
			return previousPage;
		}
		return this._getCurrenPageIndex();
	}
	_getNextMatchingPage() {
		if (this._getCurrenPageIndex() === -1) {
			const currentPage = this.f
				.getDocumentController()
				.getActivePage()
				.getPagenum();
			const nextPage = _.find(this._matchingPages, pageStr => {
				return parseInt(pageStr, 10) > currentPage;
			});
			return nextPage;
		}
		return this._getCurrenPageIndex();
	}

	previous() {
		const selectedIndex = this.getHighlightedIndex();

		if (this._getCurrenPageIndex() == -1 || (selectedIndex === 0 && this._getCurrenPageIndex() !== 0)) {
			const newMatchingPageIndex =
				this._getCurrenPageIndex() === -1 ? this._matchingPages.indexOf(this._getPreviousMatchingPage()) : this._getCurrenPageIndex() - 1;
			this._changePage(newMatchingPageIndex);
		} else {
			super.previous();
		}
	}

	next() {
		const hitCountCurrentPage = this.hitCount();
		const selectedIndex = this.getHighlightedIndex();
		if (
			this._getCurrenPageIndex() === -1 ||
			(selectedIndex === hitCountCurrentPage - 1 && this._getCurrenPageIndex() < this._matchingPages.length - 1)
		) {
			const newMatchingPageIndex =
				this._getCurrenPageIndex() === -1 ? this._matchingPages.indexOf(this._getNextMatchingPage()) : this._getCurrenPageIndex() + 1;

			this._changePage(newMatchingPageIndex);
		} else {
			super.next();
		}
	}

	_changePage(matchingPageIndex) {
		const docController = this.f.getDocumentController();
		const documentVersion = docController.getActiveDocument().getDocumentVersion();
		const page = parseInt(this._matchingPages[matchingPageIndex], 10);

		docController.changeActivePage(documentVersion.getPage(page));
	}

	hasNext() {
		if (this._getCurrenPageIndex() === -1) {
			return this._getNextMatchingPage() !== undefined;
		} else {
			return this.getHighlightedIndex() < this.hitCount() - 1 || this._getCurrenPageIndex() < this._matchingPages.length - 1;
		}
	}

	hasPrevious() {
		if (this._getCurrenPageIndex() === -1) {
			return this._getPreviousMatchingPage() !== undefined;
		} else {
			return this._i > 0 || this._getCurrenPageIndex() > 0;
		}
	}

	getNumberOfComingPages() {
		let currentPageIndex =
			this._getCurrenPageIndex() === -1 ? this._matchingPages.indexOf(this._getPreviousMatchingPage()) : this._getCurrenPageIndex();
		return this._matchingPages.length - currentPageIndex - 1;
	}

	getNumberOfPreviousPages() {
		if (this._getCurrenPageIndex() === -1) {
			const previousPageIndex = this._matchingPages.indexOf(this._getNextMatchingPage());
			if (previousPageIndex === -1) {
				const currentPage = this.f
					.getDocumentController()
					.getActivePage()
					.getPagenum();
				return currentPage < parseInt(this._matchingPages[0], 10) ? 0 : this._matchingPages.length;
			} else {
				return previousPageIndex;
			}
		}
		return this._getCurrenPageIndex();
	}
}

export default MultiPageTextSearchModel;
