import Transformation from "@esko/transformation/Transformation";

/**
 * Help functions to go from world to text coordinates and vice versa
 */

/**
 * @param {Page} page - page
 * @returns {Transformation} the transformation which can be used to transform from text to world
 */
var transform = function(page) {
	const transform = new Transformation();
	var rotation = page.getRotation();
	var isFlipped = page.isFlipped();
	transform.rotate(rotation * (Math.PI / 180));
	var box = page.getMediaBoxRect();
	var offset;

	if ((!isFlipped && rotation === 0) || (isFlipped && rotation === 90)) {
		offset = box.topleft();
	} else if ((!isFlipped && rotation === 90) || (isFlipped && rotation === 180)) {
		offset = box.topright();
	} else if ((!isFlipped && rotation === 180) || (isFlipped && rotation === 270)) {
		offset = box.bottomright();
	} else {
		offset = box.bottomleft();
	}
	transform.preTranslate(offset.x, offset.y);
	return transform;
};

/**
 * @param {Page} page - page
 * @returns {Transformation} the transformation which can be used to transform world to text
 */
var inverse = function(page) {
	return transform(page).createInverse();
};
export { transform, inverse };
