import AbstractViewerFacade from "./AbstractViewerFacade";
import { createStore, applyMiddleware } from "redux";
import createSagaMiddleware from "redux-saga";
import reduceReducers from "reduce-reducers";
import InitialStoreState from "./InitialStoreState";

/**
 * Implements AbstractViewerFacade and provides a default redux store with saga functionality
 *
 * @export ViewerFacade
 * @class
 * @extends {AbstractViewerFacade}
 */
export default class extends AbstractViewerFacade {
	constructor(...args) {
		super(...args);

		this._sagaMiddleware = createSagaMiddleware();
		this._store = createStore((state = InitialStoreState) => state, applyMiddleware(this._sagaMiddleware));
		this._reducers = [];
	}

	getStore() {
		return this._store;
	}

	addSaga(saga) {
		this._sagaMiddleware.run(saga);
	}

	addReducer(newReducer) {
		this._reducers.push(newReducer);
		this.getStore().replaceReducer(reduceReducers(InitialStoreState, ...this._reducers));
	}
}
