import { produce } from "immer";
import { ADD_ITEM, ADD_LAYOUT_MODE, SWITCH_LAYOUT_MODE, ACTIVATE_WIDGET, TOGGLE_ITEM, REMOVE_ITEM } from "./LayoutActions";
import _ from "underscore";
import { selectActiveLayoutMode } from "./LayoutSelect";

export default (state = {}, action) =>
	produce(state, newState => {
		switch (action.type) {
			case ADD_ITEM: {
				const { type, factory, name } = action.payload;
				newState.ui.items[name] = { type, getItem: _.once(() => factory("react")) };
				break;
			}

			case REMOVE_ITEM: {
				delete newState.ui.items[action.payload.name];
				break;
			}

			case ADD_LAYOUT_MODE: {
				action.payload.layoutModes.forEach((layoutMode, i) => {
					// if (i === 0) {
					// 	newState.ui.activeMode = layoutMode.getName();
					// 	layoutMode.toggle(true);
					// }
					newState.ui.modes[layoutMode.getName()] = layoutMode;
				});
				break;
			}

			case SWITCH_LAYOUT_MODE: {
				const previousLayoutMode = selectActiveLayoutMode(newState);
				if (previousLayoutMode) {
					previousLayoutMode.toggle(false);
				}
				newState.ui.activeMode = action.payload.layoutModeName;
				const newLayoutMode = selectActiveLayoutMode(newState);
				if (newLayoutMode) {
					newLayoutMode.toggle(true);
				}
				break;
			}

			case ACTIVATE_WIDGET: {
				if (newState.ui.activeWidget === action.payload.widgetName) {
					newState.ui.activeWidget = undefined;
				} else {
					newState.ui.activeWidget = action.payload.widgetName;
				}
				break;
			}

			case TOGGLE_ITEM: {
				const { itemName, enabled } = action.payload;
				newState.ui.items[itemName].disabled = !enabled;
				break;
			}
		}
	});
