import LayoutConstants from "./LayoutConstants";

const EMPTY_OBJ = {};
const EMPTY_ARR = [];

export function selectLayoutItems(store) {
	if (store.ui && store.ui.items) {
		return store.ui.items;
	}
	return EMPTY_OBJ;
}

export function selectItem(store, name) {
	return selectLayoutItems(store)[name];
}

export function selectWidget(store, name) {
	return selectItem(store, name).getItem();
}

export function selectWidgets(store) {
	const mode = selectActiveLayoutMode(store);
	if (!mode) {
		return EMPTY_ARR;
	}
	return Object.keys(selectLayoutItems(store)).filter(widgetName => {
		const item = selectItem(store, widgetName);
		if (item.type !== LayoutConstants.LayoutItemType.WIDGET) {
			return false;
		}
		const widgetType = item.getItem().getWidgetType();
		return (widgetType === "dockable" || widgetType === "maincontent") && mode.getMainPanelContent().indexOf(widgetName) === -1;
	});
}

export function selectIsItemAllowed(store, itemName) {
	const mode = selectActiveLayoutMode(store);
	const isEnabled = selectIsItemEnabled(store, itemName);
	return (!mode || mode.isItemAllowed(itemName)) && isEnabled;
}

export function selectTopbar(store) {
	const topbarWidgetName = Object.keys(selectLayoutItems(store)).filter(widgetName => {
		const item = selectItem(store, widgetName);
		return item.type === LayoutConstants.LayoutItemType.WIDGET && item.getItem().getWidgetType() === "topbar";
	})[0];
	return topbarWidgetName ? selectWidget(store, topbarWidgetName) : undefined;
}

export function selectMessagebar(store) {
	const messageBarWidgetName = Object.keys(selectLayoutItems(store)).filter(widgetName => {
		const item = selectItem(store, widgetName);
		return item.type === LayoutConstants.LayoutItemType.WIDGET && item.getItem().getWidgetType() === "messagebar";
	})[0];
	return messageBarWidgetName ? selectWidget(store, messageBarWidgetName) : undefined;
}

export function selectToolbarComponents(store) {
	const mode = selectActiveLayoutMode(store);
	if (!mode) {
		return EMPTY_ARR;
	}
	return Object.keys(selectLayoutItems(store)).filter(widgetName => {
		const isToolbarComponent = selectItem(store, widgetName).type === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT;
		return isToolbarComponent && mode.getMainPanelContent().indexOf(widgetName) === -1;
	});
}

export function selectMainPanelContent(store) {
	const mode = selectActiveLayoutMode(store);
	if (!mode) {
		return EMPTY_ARR;
	}
	return Object.keys(selectLayoutItems(store)).filter(widgetName => {
		return mode.getMainPanelContent().indexOf(widgetName) !== -1;
	});
}

export function selectLayoutModes(store) {
	return store.ui.modes ? store.ui.modes : EMPTY_OBJ;
}

export function selectLayoutModeByName(store, name) {
	return selectLayoutModes(store)[name];
}

export function selectActiveLayoutMode(store) {
	if (store.ui.activeMode != null) {
		return selectLayoutModes(store)[store.ui.activeMode];
	}
}

export function selectActiveWidget(store) {
	return store.ui.activeWidget;
}

export function selectIsItemEnabled(store, itemName) {
	return store.ui.items[itemName] ? !store.ui.items[itemName].disabled : false;
}
