import XDropModalPanelWrapper from "@esko/xdrop/panel/XDropModalPanelWrapper";
import XDropPanel from "@esko/xdrop/panel/XDropPanel";
import _ from "underscore";
import ErrorRenderer from "@esko/xdrop/renderers/ErrorRenderer";
import CompositeDataProvider from "@esko/xdrop/dataproviders/CompositeDataProvider";
import XDropDataProvider from "@esko/xdrop/dataproviders/XDropDataProvider";
import ResultEntry from "@esko/xdrop/dataproviders/ResultEntry";



class ErrorDataProvider extends XDropDataProvider {
	constructor(options) {
		super(options);
	}
	
	setQuery() {
		this.removeError();
	}

	removeError() {
		this._resultSet.clear();
		this._resultSet.commitChanges();

	}

	addError(error) {
		this._resultSet.clear();
		this._resultSet.append(new ResultEntry("error", error));
		this._resultSet.commitChanges();
	}

}

export default class DocumentBrowser {
	constructor(f) {
		this.f = f;
		this._modalOptions = {
			modalTitle: this.f.t("selectDocument"),
			modalClass: "modalWrapper viewer-app-document-browser-modal-wrapper"
		}
	}

	createDocumentsSelector(onSelected, myOptions) {
		var modalOptions = _.extend(
			this._modalOptions,
			myOptions
		);
		var options = this.f.getDocumentController().getLoadMoreDocumentsOptions();
		var { dataProvider, renderers} = options;
		renderers = _.extend({ error: ErrorRenderer}, renderers)

		const errorDataProvider = new ErrorDataProvider(); 
		dataProvider.getResultSet().on("changed", () => errorDataProvider.removeError())
		dataProvider = new CompositeDataProvider([errorDataProvider, dataProvider]);
		var xDropPanel = new XDropPanel(dataProvider, renderers, { xdropFilter: options.xdropFilter });
		var panelWrapper = new XDropModalPanelWrapper(xDropPanel, modalOptions, () => {
			panelWrapper.destroy();
			xDropPanel.getElement().remove();
		});
		

		panelWrapper.toggle(true);
		xDropPanel.on(
			"selected",
			function(entry) {
				errorDataProvider.removeError();
				Promise.resolve(onSelected(entry)).then(function(){
						panelWrapper.toggle(false);
				}).catch(e => {
					errorDataProvider.addError(e);
				});
			}.bind(this)
		);
	}

	openNewActiveDocument() {
		this.createDocumentsSelector(
			function(entry) {
				this.f.getDocumentController().openDocuments(
					entry.getData().id,
					doc => {
						if (doc) {
							this.f.getDocumentController().changeActivePage(doc.getDocumentVersionById(entry.getData().id).getPage());
						}
					},
					false
				);
			}.bind(this)
		);
	}
	openNewReferenceDocument(options) {
		this.createDocumentsSelector(
			function(entry) {
				return new Promise((resolve,reject) => {

					this.f.getDocumentController().openDocuments(
						entry.getData().id,
						function(doc) {
							if (doc) {
								var activePage = this.f.getActivePage();
								var page;
								// if the document is the same document as selected, pick another version
								let version = doc.getDocumentVersionById(entry.getData().id);
								page = version.getPage();
								
								if (page && page.canBeComparedTo(activePage)) {
									this.f.getDocumentController().changeDifferencePage(page, {isUserInitiated: true});
									resolve();
								} else if (page) {
									reject(this.f.t("CannotCompareDocuments"));
									
								}
							} else {
								reject(this.f.t("CannotLoadDocument"));
							}
						}.bind(this),
						false
						);
					})
			}.bind(this),
			options
		);
	}

	openNewStructureDocument() {
		var options = this.f.getDocumentController().getLoadMoreStructuresOptions();
		var { dataProvider, renderers } = options;
		var nofilter = options.nofilter;
		var xDropPanel = new XDropPanel(dataProvider, renderers, { xdropFilter: options.xdropFilter });
		xDropPanel.setShowFilterField(!nofilter);
		var panelWrapper = new XDropModalPanelWrapper(
			xDropPanel,
			this._modalOptions,

			() => {}
		);
		panelWrapper.toggle(true);
		xDropPanel.on(
			"selected",
			function(entry) {
				this.f.getDocumentController().openDocuments(
					entry.getData().id,
					function(doc) {
						if (doc) {
							var page = doc.getDocumentVersionById(entry.getData().id).getPage();
							this.f.getDocumentController().changeCadOverlay(page);
						}
					},
					false
				);
				panelWrapper.toggle(false);
			}.bind(this)
		);
	}
}
