import $ from "jquery";
import "./numberinput.css";

export default class PositiveNumberInput {
	constructor(f, options) {
		this.f = f;
		this.options = options;
		this.input = $("<input>", {
			class: "numberInput",
			type: "number",
			on: {
				change: e => {
					let val = e.target.value;
					if (val < 0) {
						val = 0;
						this.setValue(val);
					}
					if (options && options.onChange) {
						options.onChange(val);
					}
				}
			}
		});
		this.element = $("<div>", {
			class: "hiddenElement"
		});
		if (options.label) {
			this.element
				.append(
					$("<span>", {
						class: "alignLabel",
						text: f.t(options.label)
					})
				)
				.append(
					$("<div>", {
						class: "offsetFields"
					})
				);
		}
		if (options.value) {
			this.setValue(options.value);
		} else {
			this.setValue(0);
		}
		this.element.append(this.input);

		if (options.unit) {
			this.element.append(
				$("<span>", {
					class: "alignLabel",
					text: f.t(options.unit)
				})
			);
		}
	}

	hide() {
		this.element.addClass("hiddenElement");
	}

	show() {
		this.element.removeClass("hiddenElement");
	}

	setValue(value) {
		this.input.val(value);
	}
}
