import { selectItem, selectIsItemAllowed } from "../LayoutSelect";
import React, { useRef, useLayoutEffect } from "react";
import { useSelector } from "react-redux";
import $ from "jquery";
import useResizeObserver from "@esko/react-hooks/useResizeObserver";
import useForkRef from "@esko/react-hooks/useForkRef";
import { generateSheet } from "@esko/react-ui/css";
import classNames from "classnames";

const classes = generateSheet({
	root: {
		display: "none",
		flex: "1 1 auto",
		flexDirection: "column"
	},
	rootShown: {
		display: "flex"
	},
	widget: {
		display: "flex",
		flex: "1 1 auto",
		flexDirection: "column",
		background: "#eaeaea",
		color: "#344",
		overflow: "auto"
	},
	header: {
		display: "none",
		paddingTop: "5px",
		paddingBottom: "5px",
		backgroundColor: "#505050"
	},
	headerVisible: {
		display: "flex"
	},
	headerTitle: {
		flex: "1 1 auto",
		display: "flex",
		alignItems: "center",
		paddingLeft: "5px",
		fontWeight: "500",
		fontSize: "16px"
	},
	headerExtra: {
		display: "flex",
		alignItems: "center",
		"& .mini-btn": {
			color: "white",
			"&:hover": {
				backgroundColor: "#5a5a5a"
			}
		},
		"& .header-btns": {
			display: "flex"
		}
	}
});

const Widget = React.forwardRef(({ name, allowHeader, visible = true, style, className, onResize }, forwardRef) => {
	const item = useSelector(store => selectItem(store, name).getItem());
	const isAllowed = useSelector(store => selectIsItemAllowed(store, name));
	const [resizeRef, width, height] = useResizeObserver();

	const shouldBeShown = isAllowed && visible;

	const rootRef = useForkRef(resizeRef, forwardRef);
	const divRef = useRef();
	const headerRef = useRef();

	// useLayoutEffect is needed instead of useEffect,
	// because certain widgets expect the DOM elements to be attached to the page during execution (e.g. attributecategory)
	useLayoutEffect(() => {
		item.setContext($(divRef.current), $(headerRef.current));
	}, [item]);

	useLayoutEffect(() => {
		onResize && onResize(width, height);
		item.resized(width, height);
	}, [width, height, item, onResize]);

	return (
		<div
			className={classNames(className, classes.root, {
				[classes.rootShown]: shouldBeShown
			})}
			style={style}
			ref={rootRef}
		>
			<div className={classNames(classes.header, { [classes.headerVisible]: allowHeader })}>
				<div className={classes.headerTitle}>{item.getTitle()}</div>
				<div ref={headerRef} className={classes.headerExtra}></div>
			</div>
			<div className={classes.widget} ref={divRef} />
		</div>
	);
});

Widget.displayName = "Widget";
export default React.memo(Widget);
