import usePopper from "@esko/react-hooks/usePopper";
import { generateSheet } from "@esko/react-ui/css";
import classNames from "classnames";
import React, { useCallback, useContext, useMemo } from "react";
import { useDispatch, useSelector } from "react-redux";
import { activateWidget } from "../LayoutActions";
import { selectActiveWidget, selectWidget, selectIsItemAllowed } from "../LayoutSelect";
import LayoutContext from "./LayoutContext";
import Widget from "./Widget";

const classes = generateSheet({
	icon: {
		display: "flex",
		justifyContent: "center",
		alignItems: "center",
		width: "50px",
		height: "50px",
		cursor: "pointer",
		"&:hover": {
			backgroundColor: "rgba(0,0,0,0.1)"
		}
	},
	hidden: {
		display: "none"
	},
	iconOpen: {
		backgroundColor: "rgba(0,0,0,0.2)"
	},
	popup: {
		position: "absolute",
		top: 0,
		left: 0,
		maxHeight: "100%",
		padding: "5px"
	}
});

function WidgetBarItem({ widgetName }) {
	const dispatch = useDispatch();
	const widget = useSelector(store => selectWidget(store, widgetName));
	const open = useSelector(store => selectActiveWidget(store) === widgetName);
	const isAllowed = useSelector(store => selectIsItemAllowed(store, widgetName));
	const { mainContent } = useContext(LayoutContext);
	const popperSettings = useMemo(
		() => ({
			placement: "left-start",
			modifiers: {
				preventOverflow: {
					boundariesElement: mainContent,
					padding: 0
				},
				keepTogether: {
					enabled: false //disabled, because otherwise widgets will float over toolbar
				},
				arrow: {
					enabled: false //disabled, because it needs keepTogether
				}
			}
		}),
		[mainContent]
	);

	const [iconRef, popupRef, styles, triggerPopperResize] = usePopper(open && mainContent && isAllowed, popperSettings);
	const onClick = useCallback(() => dispatch(activateWidget(widgetName)), [dispatch, widgetName]);
	const onResize = useCallback(() => triggerPopperResize(), [triggerPopperResize]);

	return (
		<>
			<div ref={iconRef} className={classNames(classes.icon, { [classes.iconOpen]: open, [classes.hidden]: !isAllowed })} onClick={onClick}>
				<i className={widget.getIcon()} />
			</div>
			<Widget
				ref={popupRef}
				className={classes.popup}
				name={widgetName}
				visible={open}
				style={styles}
				allowHeader={true}
				onResize={onResize}
			/>
		</>
	);
}

export default React.memo(WidgetBarItem);
