import { generateSheet } from "@esko/react-ui/css";
import React from "react";
import ReactDOM from "react-dom";
import { Provider } from "react-redux";
import PrivateChannel from "../../core/PrivateChannel";
import ListenerUtil from "../../utils/ListenerUtil";
import Compare3DMode from "../../view3D/Compare3DMode";
import Collada3DMode from "../desktop/Collada3DMode";
import Desktop2D3DMode from "../desktop/Desktop2D3DMode";
import Desktop3DMode from "../desktop/Desktop3DMode";
import DesktopVideoMode from "../desktop/DesktopVideoMode";
import "../flex/less/flex.less";
import FlexBasicMode from "../flex/modes/FlexBasicMode";
import FlexWizardMode from "../flex/modes/FlexWizardMode";
import { default as ViewerLayout } from "../Layout";
import { addLayoutModes, switchLayoutMode } from "../LayoutActions";
import Layout from "./Layout";

const classes = generateSheet({
	container: {
		display: "flex",
		flexDirection: "column",
		position: "absolute",
		top: 0,
		left: 0,
		bottom: 0,
		right: 0
	}
});

export default class extends ViewerLayout {
	constructor(...args) {
		super(...args);
		PrivateChannel.installTo(this, this.f, "layout.react");
		ListenerUtil.mixin(this, this.f);

		// Render React tree in separate div, because too many modules rely on f.getContext() to append hidden divs,
		// Because ReactDOM.render empties the div first, removing any existing dom elements
		const container = document.createElement("div");
		container.classList.add(classes.container);
		this._context.append(container);
		ReactDOM.render(
			<Provider store={this.f.getStore()}>
				<Layout />
			</Provider>,
			container
		);

		this.f.dispatch(addLayoutModes(this._getLayoutModes()));
		this.f.dispatch(switchLayoutMode(this._getLayoutModes()[0].getName()));
		this.f.emit("layout.finishRegisteringModes");
	}

	_getLayoutModes() {
		return [
			new FlexBasicMode(this.f, this),
			new FlexWizardMode(this.f, this),
			new Collada3DMode(this.f, this),
			new Desktop2D3DMode(this.f, this),
			new Desktop3DMode(this.f, this),
			new DesktopVideoMode(this.f, this),
			new Compare3DMode(this.f, this)
		];
	}

	isItemAllowed() {
		return true;
	}
}
