import ViewportPopoverModel from "@esko/web-viewer/viewport/model/ViewportPopoverModel";
import tpl from "@esko/web-viewer/svg/FontSelectionPopoverTemplate.tpl.html";
import "./less/svg.less";
import Template from "@esko/template";
import ShapeUtils from "@esko/shapeutils";
import $ from "jquery";
/**
 * PoveroverModel for text selection tool
 *
 * @exports FontSelectionPopoverModel
 * @memberof SvgModule
 * @class FontSelectionPopoverModel
 * @augments Viewport.Model.ViewportPopoverModel
 * @constructor
 *
 * @param {Core.ViewerFacade} f
 * @param {String} name
 */
var FontSelectionPopoverModel = function(f, name, layerModel) {
	ViewportPopoverModel.call(this, f, name);

	this._content = $("<div/>").addClass("measureInfo viewerApp--font-selection-popover");
	this._layerModel = layerModel;
	this._layerModel.on("modelChanged", () => this.update());
	// initialize template
	this.t = new Template(f.i18n, tpl);
	this.t.setContext(this._content);

	// load language if needed
	this.f.i18n.loadBundle("viewer", () => {
		this.update();
	});
	// Add a regular annotation
	this.t.bind("mousedown", ".addAnnotation", (source, event) => {
		this.f.requestModule("annotation", annotationModule => {
			annotationModule.promptIfAnnotationOutsidePage(
				this.getFrame(),
				() => {
					// add annotation

					var font = this._layerModel.getFont();
					this.f.emit(
						"module.annotation.createAnnotation",
						this.getFrame(),
						font.fontFamily ? `${f.t("Font")}: ${font.fontFamily} \n ${f.t("FontSize")}: ${font.fontSize}` : `${f.t("FontSize")}: ${font.fontSize}`
					);
					this._emit("addAnnotation");
				},
				function() {
					// do nothing when 'No' is pressed
				},
				false,
				true
			);
		});
		this._emit("popupClosed");
	});

	this.getFrame = function() {
		var font = this._layerModel.getFont();
		var point = layerModel.getPoint();

		if (font) {
			return font.rect;
		} else if (point) {
			return new ShapeUtils.Rect(point, new ShapeUtils.Size(0.001, 0.001));
		} else {
			return new ShapeUtils.Rect(new ShapeUtils.Point(0, 0), new ShapeUtils.Size(1, 1));
		}
	};

	this.getContent = function() {
		return this._content;
	};

	this.getPreferredPosition = function() {
		// override
		return "right";
	};

	this.getRestrictBorder = function() {
		return 0;
	};

	this.update();
};

// inherit
FontSelectionPopoverModel.prototype = Object.create(ViewportPopoverModel.prototype);
FontSelectionPopoverModel.prototype.constructor = FontSelectionPopoverModel;

/**
 * Update the popover
 */
FontSelectionPopoverModel.prototype.update = function() {
	var font = this._layerModel.getFont();
	if (font) {
		var { fontFamily, fontSize, text } = font;
		this.t.d.fontFamily = fontFamily;
		this.t.d.fontSize = fontSize;
		this.t.d.text = text;
	} else {
		this.t.d.fontFamily = null;
	}
	this.t.render();
};

/**
 * Set the location of the popover
 * @param {ShapeUtils.Point} point for which the popover should be shown
 */
FontSelectionPopoverModel.prototype.setPopoverLocation = function(point) {
	this._popoverLocation = new ShapeUtils.Rect(point, new ShapeUtils.Size(0.001, 0.001));
};

export default FontSelectionPopoverModel;
