import { SelectionModel } from "./TextSelectionLayerModel";
import _ from "underscore";

/**
 * Selection Model used by the text selection tool.
 * It listens to the selections from search and 'text select' models and forwards the selection made by the last changed
 * @param {TextSearchModel} searchModel
 * @param {TextStartEndPointModel} startEndModel
 */
export default class extends SelectionModel {
	constructor(f, searchModel, startEndModel) {
		super(f, "TextSelectionToolSelectionModel");
		this._lastChanged = searchModel;
		_.each([searchModel, startEndModel], model =>
			model.on("changed.*", () => {
				this._lastChanged = model;
				this._emit("changed.selection");
			})
		);
	}
	getSelection() {
		return this._lastChanged.getSelection();
	}
	getContent() {
		return this._lastChanged.getContent();
	}
	getHighlight() {
		return this._lastChanged.getHighlight();
	}
}
