import Mousetrap from "@esko/mousetrap";
import _ from "underscore";

class KeyboardUtils {
	listenerObjects = [];

	/**
	 * Bind single key using MouseTrap
	 * @param {String} shortcut - keycombo
	 * @param {function} callback
	 * @param {String} option - keyup|keydown|keypress
	 */
	bind(shortcut, callback, option = "keydown") {
		Mousetrap.bind(shortcut, callback, option);
		this.listenerObjects.push({ shortcut, callback, option });
	}

	/**
	 * Unbind single key
	 */
	unbind(shortcut, callback, option = "keydown") {
		Mousetrap.unbind(shortcut, callback, option);
		this.listenerObjects = _.filter(this.listenerObjects, obj => {
			return !_.isEqual(obj, { shortcut, callback, option });
		});
	}
	/**
	 * Bind keys using MouseTrap
	 * @param {Array} array - array of arrays [keycombo, function, keyup|keydown|keypress]
	 */
	bindKeys(array) {
		for (var i = 0; i < array.length; i++) {
			this.bind(array[i][0], array[i][1], array[i][2]);
		}
	}

	/**
	 * Unbind keys
	 * @param array
	 */
	unbindKeys(array) {
		for (var i = 0; i < array.length; i++) {
			this.unbind(array[i][0], array[i][1], array[i][2]);
		}
	}

	/**
	 * Unbind all listeners
	 */
	destroy() {
		this.listenerObjects.forEach(obj => {
			Mousetrap.unbind(obj.shortcut, obj.callback, obj.option);
		});
		this.listenerObjects = [];
	}
}

export default KeyboardUtils;
