import CompositeDataProvider from "./CompositeDataProvider";

class ChainedDataProvider extends CompositeDataProvider {
    constructor(...args){
        super(...args);
    }

    filter(filter) {
        return new Filter(this, filter);
    }

    map(map) {
        return new Map(this, map);
    }

    ifEmpty(ifEmpty) {
        return new IfEmpty(this, ifEmpty);
    }


}
class IfEmpty extends ChainedDataProvider {
	constructor(dataprovider, ifEmpty = query => []) {
		super(dataprovider);
		this.ifEmpty = ifEmpty;
	}

	_updateResultSetImpl() {
    let results = this._generateResultList();
    if(results.length === 0) {
      results = this.ifEmpty(this._query)
    }
		this._resultSet.setEntries(results);
		this._resultSet.commitChanges();		
	}
}


class Filter extends ChainedDataProvider {
	constructor(dataprovider, filter = entry => true) {
		super(dataprovider);
		this.filter = filter;
	}

	_updateResultSetImpl() {
        let results = this._generateResultList().filter(this.filter);
		this._resultSet.setEntries(results);
		this._resultSet.commitChanges();		
	}
}

class Map extends ChainedDataProvider {
	constructor(dataprovider, map = entry => true) {
		super(dataprovider);
		this.map = map;
	}

	_updateResultSetImpl() {
        let results = this._generateResultList().map(this.map);
		this._resultSet.setEntries(results);
		this._resultSet.commitChanges();		
	}
}

export const filter = (dataProvider, filter) => new Filter(dataProvider, filter);
export const map = (dataProvider, map) => new Map(dataProvider, map);
export const ifEmpty = (dataProvider, ifEmpty) => new IfEmpty(dataProvider, ifEmpty);

export default ChainedDataProvider;
